@@ -28,3 +28,15 @@ from .api import advanced_features
 from .api import trend_tweets_features
 from .api import filter_possible_irregular_profiles
 from .api import evaluate_NLP_profiles
+from .api import general_pieChart
+from .api import sentiental_analysis_features
+from .api import distribution_sentiment_tweets
+from .api import tokenize_tweets
+from .api import keywords_score
+from .api import hashtag_score
+from .api import terrorist_keywords_belonging
+from .api import pie_chart_keywords_terrorist
+from .api import terrorist_hastags_belonging
+from .api import pie_chart_hashtags_terrorist
+from .api import NLP_analysis
+from .api import more_tweets_required
\ No newline at end of file
