@@ -9,7 +9,6 @@ import subprocess
 import sys
 import warnings
 warnings.filterwarnings("ignore")
-
 #---------------------------------------------------------------INSTALLATION---------------------------------------------------------#
 
 def install(package):
@@ -22,6 +21,8 @@ install ("urllib3")
 install("requests")
 install("matplotlib")
 install("tweepy")
+install("nltk")
+install("textblob")
 ##----------------------------------------------IMPORTS---------------------------------------------------------------------------#	
 import json
 import pandas as pd
@@ -36,6 +37,240 @@ import tweepy
 from os import path
 from datetime import timedelta
 
+#---------------------------------------------NLP-Part-------------------------------------------------------------------------------#
+from textblob import TextBlob
+import nltk
+from nltk import RegexpTokenizer
+# some small toolkits to download
+nltk.download("wordnet")
+nltk.download('brown')
+nltk.download('punkt')
+
+def sentiental_analysis_features(data):
+	corpus =[]
+	polarities=[]
+	subjectivities=[]
+	for tweet in data.text:
+		corpus.append(TextBlob(tweet.lower()))
+	for i in range(len(corpus)):
+		polarities.append(corpus[i].polarity)
+		subjectivities.append(corpus[i].subjectivity)
+	data['polarity']= polarities
+	data['subjectivity']= subjectivities
+	return data
+
+def distribution_sentiment_tweets(data)
+	### VISUALIZATION
+	plt.rcParams['figure.figsize'] = [50, 40]
+	for i in range (len(data)):
+		x=data.polarity.iloc[i]
+		y=data.subjectivity.iloc[i]
+		if x>0.1 : # blue   red
+			if y>0.6:  # yellow   green
+				plt.scatter(x,y, color='blue')
+			else: 
+				plt.scatter(x,y, color='pink')
+		elif -0.1<=x<=0.1:
+			if 0.4<=y<=0.6:  # yellow   green
+				plt.scatter(x,y, color='gold')
+		else:
+			if y>0.6:  # yellow   green
+				plt.scatter(x,y, color='purple')
+			else: 
+				plt.scatter(x,y, color='red')  
+    plt.text(x+0.01, y+0.01, 'tweet'+str(i), fontsize=10)
+    plt.xlim(-1,1)
+    plt.ylim(0,1)
+	plt.title('sentiment anaysis',fontsize=20)
+	plt.xlabel('<------------------ NEGATIVE ------------------   POLARITY ------------------ POSITIVE ------------------>',fontsize=15)
+	plt.ylabel('<----------------- FACTS ------------------ SUBJECTIVITY ------------------ OPINIONS ----------------->',fontsize=15)
+	plt.show()
+
+def general_pieChart(data):
+	p=0 # positive
+	n=0 # Negative
+	o=0 # neutral
+	for i in range(len(data)):
+		if data['polarity'].iloc[i]>0.1:
+			p+=1
+		elif data['polarity'].iloc[i]<-0.1:
+			n+=1
+		else:
+			o+=1
+	f=0
+	op=0
+	kn=0
+	for i in range(len(data)):
+	  if data['subjectivity'].iloc[i]>0.6:
+		  op+=1
+	  elif data['subjectivity'].iloc[i]<0.4:
+		  f+=1
+	  else:
+		  kn+=1
+	plt.rcParams['figure.figsize'] = [10,5]
+	labels = 'Positive', 'Negative','Neutral'
+	sizes = [p,n,o]
+	colors = ['blue','red', 'gold'] #, 'yellowgreen', 'lightcoral','lightskyblue',
+	explode = (0, 0,0)  # explode 1st slice
+
+	labels2='Opinion','Fact','Neutral'
+	sizes2=[f,op,kn]
+	colors2=['pink','red','gold']
+	explode2=(0,0,0)
+
+	fig, (ax1, ax2) = plt.subplots(1, 2)
+	# Plot
+	ax1.pie(sizes, explode=explode, labels=labels, colors=colors, autopct='%1.1f%%', shadow=True, startangle=140)
+	ax2.pie(sizes2, explode=explode2, labels=labels2, colors=colors2, autopct='%1.1f%%', shadow=True, startangle=140)
+	plt.title('The General Sentiment Analysis of all The tweets of ')
+	plt.axis('equal')
+	plt.show()
+	
+def tokenize_tweets(data)
+	sample_data_tokenized=[]  
+	tokenizer=RegexpTokenizer(r'\w+')
+	for tweets in data.text:
+		sample_data_tokenized.append(tokenizer.tokenize(tweets.lower()))
+	return sample_data_tokenized
+	
+def keywords_score(data,sample_data_tokenized, keyword_corpus):
+	scores = []
+	for i in range(len(sample_data_tokenized)):
+		score=0
+		for c in keyword_corpus.keywords:
+			for t in sample_data_tokenized[i]:
+				if c==t:
+					score+=1
+		try:
+			scores.append(score/len(sample_data_tokenized[i]))
+		except ZeroDivisionError:
+			scores.append(0)
+    data['keyword_score']=scores
+    return data
+
+def hashtag_score(data, sample_data_tokenized, hashtag_corpus):
+	scores = []
+	for i in range(len(sample_data_tokenized)):
+		score=0
+		for c in hashtag_corpus.hashtags:
+			for t in sample_data_tokenized[i]:
+				if c==t:
+					score+=1
+		try:
+			scores.append(score)
+		except ZeroDivisionError:
+			scores.append(0)
+	data['hashtags_score'] = scores
+	data['index'] = [i for i in range(len(data))]
+	data['names']= ['TWEET '+str(i) for i in range(len(data))]
+	return data	
+	
+def terrorist_keywords_belonging(data):
+	temp = data[data['keyword_score']>0].reset_index(drop=True)
+	# Draw plot
+	fig, ax = plt.subplots(figsize=(36,8), dpi= 80)
+	ax.vlines(x=temp.index, ymin=0, ymax=temp.keyword_score, color='firebrick', alpha=0.7, linewidth=2)
+	ax.scatter(x=temp.index, y=temp.keyword_score, s=75, color='firebrick', alpha=0.7)
+	# Title, Label, Ticks and Ylim
+	ax.set_title('Lollipop Chart for The belonging each tweet to the Terroristic Cluster', fontdict={'size':22})
+	ax.set_ylabel('Belonging Score of each tweet to the cluster1' )
+	ax.set_xticks(temp.index)
+	ax.set_xticklabels(temp.names.str.upper(), rotation=90, fontdict={'horizontalalignment': 'right', 'size':12})
+	ax.set_ylim(0, 1)
+	# Annotate
+	for row in temp.itertuples():
+		ax.text(row.Index, row.keyword_score+0.15,rotation=90, s=round(row.keyword_score, 2), horizontalalignment= 'center', verticalalignment='bottom', fontsize=14)
+	plt.show()
+	
+def pie_chart_keywords_terrorist(data):
+	data_terr = data[data['keyword_score']>0].reset_index(drop=True)
+	p=0
+	n=0
+	o=0
+	for i in range(len(data_terr)):
+		if data_terr['polarity'].iloc[i]>0.1:
+			p+=1
+		elif data_terr['polarity'].iloc[i]<-0.1:
+			n+=1
+		else:
+			o+=1
+	f=0
+	op=0
+	for i in range(len(data_terr)):
+		if data_terr['subjectivity'].iloc[i]>0.5:
+			op+=1
+		else:
+			f+=1
+	labels = 'Positive', 'Negative','Neutral'
+	sizes = [p,n,o]
+	colors = ['yellowgreen','red', 'gold'] #, 'yellowgreen', 'lightcoral','lightskyblue',
+	explode = (0, 0,0)  # explode 1st slice
+	labels2='Opinion','Fact'
+	sizes2=[f,op]
+	colors2=['lightblue','yellow']
+	explode2=(0,0)
+	fig, (ax1, ax2) = plt.subplots(1, 2)
+	# Plot
+	ax1.pie(sizes, explode=explode, labels=labels, colors=colors, autopct='%1.1f%%', shadow=True, startangle=140)
+	ax2.pie(sizes2, explode=explode2, labels=labels2, colors=colors2, autopct='%1.1f%%', shadow=True, startangle=140)
+	plt.title('from '+str(len(data))+' tweets, '+ str(len(data_terr))+' tweets are belonging to Terroristic contents')
+	plt.axis('equal')
+	plt.show()
+	
+
+def terrorist_hashtags_belonging(data):
+	temp = data[ data['hashtags_score']>0].reset_index(drop=True)
+	# Draw plot
+	fig, ax = plt.subplots(figsize=(36,24), dpi= 80)
+	ax.vlines(x=temp.index, ymin=0, ymax=temp.hashtags_score, color='firebrick', alpha=0.7, linewidth=2)
+	ax.scatter(x=temp.index, y=temp.hashtags_score, s=75, color='firebrick', alpha=0.7)
+	# Title, Label, Ticks and Ylim
+	ax.set_title('Lollipop Chart for The belonging each tweet to the Terrorist Hashtag Network', fontdict={'size':22})
+	ax.set_ylabel('Number of Terrorist Hashtags' )
+	ax.set_xticks(temp.index)
+	ax.set_xticklabels(temp.names.str.upper(), rotation=90, fontdict={'horizontalalignment': 'right', 'size':12})
+	ax.set_ylim(0, 30)
+	# Annotate
+	for row in temp.itertuples():
+		ax.text(row.Index+0.01, row.hashtags_score+0.15,rotation=90, s=round(row.hashtags_score, 2), horizontalalignment= 'center', verticalalignment='bottom', fontsize=14)
+	plt.show()
+
+def pie_chart_hashtags_terrorist(data):
+	temp = data[data['hashtags_score']>0].reset_index(drop=True)  
+	p=0
+	n=0
+	o=0
+	for i in range(len(temp)):
+		if temp['polarity'].iloc[i]>0.1:
+			p+=1
+		elif temp['polarity'].iloc[i]<-0.1:
+			n+=1
+		else:
+			o+=1
+	f=0
+	op=0
+	for i in range(len(temp)):
+		if temp['subjectivity'].iloc[i]>0.5:
+			op+=1
+		else:
+			f+=1
+	
+	labels = 'Positive', 'Negative','Neutral'
+	sizes = [p,n,o]
+	colors = ['yellowgreen','red', 'gold'] #, 'yellowgreen', 'lightcoral','lightskyblue',
+	explode = (0, 0,0)  # explode 1st slice
+	labels2='Opinion','Fact'
+	sizes2=[f,op]
+	colors2=['lightblue','yellow']
+	explode2=(0,0)
+	fig, (ax1, ax2) = plt.subplots(1, 2)
+	# Plot
+	ax1.pie(sizes, explode=explode, labels=labels, colors=colors, autopct='%1.1f%%', shadow=True, startangle=140)
+	ax2.pie(sizes2, explode=explode2, labels=labels2, colors=colors2, autopct='%1.1f%%', shadow=True, startangle=140)
+	plt.title('from '+str(len(data))+' tweets, '+ str(len(temp))+' tweets are following Terroristic Hashtags')
+	plt.axis('equal')
+	plt.show()  
+
 #----------------------------------------FUNCTIONS-----------------------------------------------------------------------------#
 
 def load_file(df):
@@ -712,18 +947,38 @@ def filter_possible_irregular_profiles(data_join):
 
     return data_join
 
+def NLP_analysis(data, keyword_corpus, hashtag_corpus):
+	tweets = data["text_tweets_days_list"]
+	tweets_join = [j for i in tweets for j in i]
+	tweets_df = pd.DataFrame(tweets_join)
+	tweets_df = tweets_df[tweets_df["RT"] !=2].reset_index(drop=True)
+	tweets_df = sentiental_analysis_features(tweets_df)
+	sample_data_tokenized = tokenize_tweets(tweets_df)
+	tweets_df = keyword_score(tweets_df, sample_data_tokenized,keyword_corpus)
+	tweets_df = hashtag_score(tweets_df, sample_data_tokenized,hashtag_corpus)	
+	tweets_df["terrorist_suggestion"] = 0
+	tweets_df["terrorist_suggestion"][(tweets_df["hashtag_score"]>0) | (tweets_df["keyword_score"]>0)] = 1
+	data["per_hash_key"] = len(tweets_df[tweets_df["terrorist_suggestion"]==1]) / len(tweets_df[tweets_df["terrorist_suggestion"]==0])
+	tweets_df["terrorist_suggestion"][(tweets_df["terrorist_suggestion"]==1) & (tweets_df["polarity"] < -0.1) & (tweets_df["subjectivity"]>0.6)]= 2
+	data["per_terr_hash_key"] = len(tweets_df[tweets_df["terrorist_suggestion"]==2]) / len(tweets_df[tweets_df["terrorist_suggestion"]==1])
+	data["suggestion"] = "No terrorist"
+	data["suggestion"][((data["per_terr_hash_key"]) >0.7) & (data["per_hash_key"]>0.15))] = "Terrorist"
 	
-def evaluate_NLP_profiles(df):
-	df_evaluate = df[(df["evaluate_NLP"] !="No") & df["evaluate_NLP"] !="More Data"]
-	node_list_moreData = list(df[df["evaluate_NLP"]=="More Data"])
+	return data,tweets_df
+	
+def more_tweets_required(df,consumerKey,consumerSecret,accessToken,accessTokenSecret):
+	df_evaluate = df[(df["evaluate_NLP"] !="No") & (df["evaluate_NLP"] !="More Data")]
+	node_list_moreData = list(df[df["evaluate_NLP"]=="More Data"]["screen_name"])
 	if len(node_list_moreData) >0:
 		df_metadata_moreData, list_more_Data_noData = user_timeline_extraction(node_list_moreData,consumerKey,consumerSecret,accessToken,accessTokenSecret,5,"fichero_guardar_noData.csv")
-		df_metadata_moreData = select_possible_fakes(df_metadata_moreData)
-		df_metadata_moreData = df_metadata_moreData[(df_metadata_moreData["evaluate_NLP"] !="No") & df_metadata_moreData["evaluate_NLP"] !="More Data"]
+		df_metadata_moreData = filter_possible_irregular_profiles(df_metadata_moreData)
+		df_metadata_moreData = df_metadata_moreData[(df_metadata_moreData["evaluate_NLP"] !="No") & (df_metadata_moreData["evaluate_NLP"] !="More Data")]
 		df_evaluate = df_evaluate.append(df_metadata_moreData, ignore_index=True)
-	for i in range(len(df_evaluate)):
-		tweets = df_evaluate.iloc[i]["text_tweets_days_list"]
-		tweets_join = [j for i in tweets for j in i]
-		tweets_df = pd.DataFrame(tweets_join)
-		tweets_df.to_csv("tweets_profile_example.csv",index=False)
-	return df_evaluate
\ No newline at end of file
+	return df_evaluate
+
+	
+def evaluate_NLP_profiles(data,keyword_corpus, hastag_corpus):	
+	for i in range(len(data)):
+		profile_evaluated, tweets_df = NLP_analysis(data.iloc[i], keyword_corpus, hashtags_corpus)
+		data.iloc[i] = profile_evaluated	
+	return data
\ No newline at end of file
